import java.io.FileWriter;
import java.io.IOException;
import java.util.Random;

/**
 * 
 * @author Esteve Mir Esprrech
 * v1.0 November 2015
 * Test data generator
 * 
 */
public class dataGenerator {
	
	
	private static final String COMMA_DELIMITER = ",";
	private static final String NEW_LINE_SEPARATOR = "\n";
	
	private int nFlights;
	private int days;
	private String[] airports;
	private String[][] flights;
	private int [][] travels;
	
	
	public static void main(String []args){
		dataGenerator program = new dataGenerator(10,500,5);
		program.generateTravels();
		program.generateFlights();
		program.proof();
		program.generateFile("P12.csv");
	}
	
	
	/**
	 * Function to initialize data structure to store test data
	 * of flights, airports and days.
	 * 
	 * @param airports: number of airports to generate
	 * @param flights: number of flights to generate
	 * @param days: number of days for flights
	 */
	public dataGenerator(int airports, int flights, int days){
		this.airports = new String[airports];
		this.nFlights = flights;
		this.days = days;
		this.flights = new String[flights][6];
		this.travels = new int[airports][airports];
		for(int i=0; i<airports; i++){
			this.airports[i] = "AIR"+i;
		}
	}
	
	/**
	 * Function to generate file with comma separated test data
	 * @param fileName: Name of the file to generate
	 */
	public void generateFile (String fileName){
		FileWriter fileWriter = null;
		
		try {
			fileWriter = new FileWriter(fileName);
			
			//Write a new student object list to the CSV file
			for (int i =0; i<nFlights;i++) {
				fileWriter.append(flights[i][0]);
				fileWriter.append(COMMA_DELIMITER);
				fileWriter.append(flights[i][1]);
				fileWriter.append(COMMA_DELIMITER);
				fileWriter.append(flights[i][2]);
				fileWriter.append(COMMA_DELIMITER);
				fileWriter.append(flights[i][3]);
				fileWriter.append(COMMA_DELIMITER);
				fileWriter.append(flights[i][4]);
				fileWriter.append(COMMA_DELIMITER);
				fileWriter.append(flights[i][5]);
				fileWriter.append(NEW_LINE_SEPARATOR);
			}
			
			System.out.println("CSV file creation ok");
			
		} catch (Exception e) {
			System.out.println("Error in writing csv");
			e.printStackTrace();
		} finally {
			
			try {
				fileWriter.flush();
				fileWriter.close();
			} catch (IOException e) {
				System.out.println("Error in fileWriter");
                e.printStackTrace();
			}
			
		}
	}
	
	/**
	 * Function that will generate all the flights based on the information stored on the 
	 * class variables:
	 *  private int nFlights;
	 *	private int days;
	 *	private String[] airports;
	 *	private String[][] flights;
	 *	private int [][] travels;
	 */
	public void generateFlights(){
		Random rand = new Random();
		String h, m, h2, m2;
		
		for (int i=0;i<this.nFlights;i++){
		    int hour = rand.nextInt(24);
		    int minute = rand.nextInt(60);
		    
		    if(hour<10) h = "0"+hour;
		    else h=""+hour;
		    if(minute<10) m="0"+minute;
		    else m =""+minute;
		    
		    int firstAirport = rand.nextInt(this.airports.length);
		    int lastAirport = rand.nextInt(this.airports.length);
		    while (lastAirport==firstAirport) lastAirport = rand.nextInt(this.airports.length);
		    
		    int firstDay = rand.nextInt(this.days);
		    int lastDay = firstDay;
		    
		    flights[i][0]= airports[firstAirport];
		    flights[i][1] = ""+firstDay;
		    flights[i][2] = h+":"+m;

		    int travelTime = travels[firstAirport][lastAirport];
		    int hour2 = hour+(travelTime/60);
		    int minute2 = minute+(travelTime % 60);
		    
			if (minute2>59){
				hour2+=1; 
				minute2-=60;
			}
			if (hour2>23){
				lastDay+=1;
				hour2 -=24;
			}
		    
		    if(hour2<10) h2= "0"+hour2;
		    else h2=""+hour2;
		    if(minute2<10) m2="0"+minute2;
		    else m2 =""+minute2;
		    
		    flights[i][3] = airports[lastAirport];
		    flights[i][4] = ""+lastDay;
		    flights[i][5] = h2+":"+m2;
		    
		}
	}
	
	/**
	 * Function that calculates and stores travels
	 */
	public void generateTravels(){
		Random rand = new Random();
		
		for (int i=0; i<airports.length;i++){
			for (int j=0; j<airports.length;j++){
				if (i==j) travels[i][j]=0;
				else {
					int randomTravelTime = rand.nextInt(240 - 60 + 1) + 60;
					travels[i][j] = randomTravelTime;
					travels[j][i] = randomTravelTime;
				}
			}
		}
	}
	
	
	/**
	 * Function that prints data generated in a human readable form.
	 */
	public void proof(){
		for (int i=0;i<this.nFlights;i++){
			for (int j=0;j<6;j++){
				System.out.print(flights[i][j]+" -- ");
			}
			System.out.println();
		}
	}
}
