import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * 
 * @author Esteve Mir Esprrech
 * v1.0 November 2015
 * This class stores all the parameters of the ACSP problem for FAS algorithm
 * 
 */
public class ACSP {
	
	//Constrains 
	public static final int MIN_SITTING_TIME_BETWEEN_FLIGHTS = 30;
	public static final int MIN_DESC_TIME = 10;
	public static final int MAX_FLYING_TIME = 11*60;
	public static final int MIN_REST_TIME_AFTER_MORE_THAN_12HOURS = 24*60;
	public static final int MAX_SITTING_TIME_BETWEEN_FLIGHTS = 50;
	public static final int MAX_WORKING_TIME = 14*60;
	public static final int MAX_FLIGHTS_IN_A_DUTY = 8;
	
	public static final double BASIC_PAYMENT_PER_SERVICE = 300;
	public static final double BONUS_PER_EXTRA_MINUTE_OF_FLIGHT = 1.5;
	public static final double TOTAL_HOTEL_COST = 30;
	public static final double BONUS_PER_EXTRA_MINUTE_OF_REST = 0.1;
	public static final double TOTAL_COST_PER_LOST_MINUTE = 0.2;
	public static final int MIN_PAID_FLY_TIME = 5*60;
	
	private List<flight> flights; //Flights
	
	/**
	 * Main function 
	 * @param args:
	 * 			Arguments for the function
	 */
	public static void main(String []args){
		
		long start = 0;
		int executions = 0;
		
		while (executions<35){
			ACSP newProblem = new ACSP();
			newProblem.parseData();
			forgetfulAntSystem problem = new forgetfulAntSystem(1,5,0.3,0.05,newProblem.getFlights(), 100, 2000, 100);
			start = System.nanoTime();
			problem.init();
		    double time = (System.nanoTime() - start)/ 1000000000.0;
			System.out.println(time);
			executions++;
		}
	}
	
	
	/**
	 * Function to parse problem data from csv file
	 */
	public void parseData(){
		this.flights = new ArrayList<flight>();
		
		String csvFile = "C:/Users/TREEME/workspace/DATAGENERATOR/P12.csv";
		BufferedReader br = null;
		String line = "";
		String cvsSplitBy = ",";
		int index =0;
		int departureTime;
		int departureDay;
		int arrivalTime;
		int arrivalDay;
		
		try {

			br = new BufferedReader(new FileReader(csvFile));
			while ((line = br.readLine()) != null) {

			        // use comma as separator
				String[] flight = line.split(cvsSplitBy);
				
				departureTime = (Integer.parseInt(flight[2].substring(0, 2))*60)+Integer.parseInt(flight[2].substring(3));
				arrivalTime = (Integer.parseInt(flight[5].substring(0, 2))*60)+Integer.parseInt(flight[5].substring(3));
				departureDay = Integer.parseInt(flight[1]);
				arrivalDay = Integer.parseInt(flight[4]);
				flights.add(new flight(index, flight[0], flight[3], departureTime,arrivalTime,departureDay,arrivalDay));
				index++;
			}

		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			if (br != null) {
				try {
					br.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}
	
	public List<flight> getFlights(){
		return this.flights;
	}

}
