/**
 * 
 * @author Esteve Mir Esprrech
 * v1.0 November 2015
 * This class stores all the information of a flight
 * 
 */
public class flight {
	
	private int ID;
	private String departureCity;
	private String destinationCity;
	private int departureTime;
	private int departureDay;
	private int arrivalTime;
	private int arrivalDay;
	
	/**
	 * CONSTRUCTOR 1
	 * 
	 * @param ID:
	 * 		Flight ID
	 * @param departureCity:
	 * 		City from where the flight will depart
	 * @param destinationCity:
	 * 		Destination city for the flight
	 * @param departureTime:
	 * 		Time at which the flight will depart
	 * @param arrivalTime:
	 * 		Time at which the flight will arrive to destination
	 * @param departureDay:
	 * 		Day at which the flight will depart
	 * @param arrivalDay:
	 * 		Day at which the flight will arrive to destination
	 */
	public flight(int ID, String departureCity, String destinationCity, int departureTime, int arrivalTime, 
				int departureDay, int arrivalDay){
		this.ID = ID;
		this.departureCity = departureCity;
		this.destinationCity = destinationCity;
		this.departureTime = departureTime;
		this.arrivalTime = arrivalTime;
		this.departureDay = departureDay;
		this.arrivalDay = arrivalDay;
	}
	
	/**
	 * CONSTRUCTOR 2
	 * @param newFlight:
	 * 		Flight to clone
	 */
	public flight(flight newFlight){
		this.ID = newFlight.getID();
		this.departureCity = newFlight.getDepartureCity();
		this.destinationCity = newFlight.getDestinationCity();
		this.departureTime = newFlight.getDepartureTime();
		this.arrivalTime = newFlight.getArrivalTime();
		this.departureDay = newFlight.getDepartureDay();
		this.arrivalDay = newFlight.getArrivalDay();
	}
	
	
/******************************************GETTERS AND SETTERS*********************************************************************/
	
	/**
	 * 
	 * @return: The departure city for the actual flight
	 */
	public String getDepartureCity(){
		return this.departureCity;
	}
	
	/**
	 * 
	 * @return: The destination city for the actual flight
	 */
	public String getDestinationCity(){
		return this.destinationCity;
	}
	
	/**
	 * 
	 * @return: The departure time for the actual flight
	 */
	public int getDepartureTime(){
		return this.departureTime;
	}
	
	/**
	 * 
	 * @return: The arrival time for the actual flight
	 */
	public int getArrivalTime(){
		return this.arrivalTime;
	}
	
	/**
	 * 
	 * @return: The departure day for the actual flight
	 */
	public int getDepartureDay(){
		return this.departureDay;
	}
	
	/**
	 * 
	 * @return: The arrival day for the actual flight
	 */
	public int getArrivalDay(){
		return this.arrivalDay;
	}
	
	/**
	 * 
	 * @return: Actual flight ID
	 */
	public int getID(){
		return this.ID;
	}

	
	/*******************************************HELPER FUNCTIONS****************************************************************/
	
	
	/**
	 * Function to return the time difference between this flight and an early one.
	 *  
	 * @param earlyFlightArrivalTime:
	 * 		Time at which the early flight arrived.
	 * @param earlyFlightArrivalDay:
	 * 		Day at which the early flight arrived.
	 * @return:
	 * 		Time difference between this flight and an early one
	 */
	public int returnDifferenceTime(int earlyFlightArrivalTime, int earlyFlightArrivalDay){
		int dayDifference;
		if (earlyFlightArrivalDay<this.arrivalDay){
			dayDifference = (1439-earlyFlightArrivalTime)+this.departureTime;
		} else {
			dayDifference = 0;
		}
		int timeDifference = this.departureTime - earlyFlightArrivalTime;
		return timeDifference+dayDifference;
	}
	
	/**
	 * 
	 * @return:
	 * 		The flying time for this flight
	 */
	public int getFlyingTime(){
		return (this.arrivalTime-this.departureTime);
	}
	
	public String toString(){
		String information ="";
		information +="    Flight "+this.ID+" - From "+this.departureCity+" to "+this.destinationCity;
		information +="            Departure: "+this.departureDay+" on "+this.departureTime;
		information +="            Arrival:   "+this.arrivalDay+" on "+this.arrivalTime;
		
		return information;
	}
}
